﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Cci;
using System.Drawing;

namespace AndersLiu.Reflector.Program.UI.AssemblyTreeNode
{
    class ErrorNodeTag<TUITreeNode> :
        IAssemblyTreeNode<TUITreeNode>,
        IWorkspaceUnitNodeTag
        where TUITreeNode : class
    {
        #region IAssemblyTreeNode<TUITreeNode>
        public TUITreeNode UITreeNode { get; private set; }

        public NodeType NodeType
        {
            get { return NodeType.Error; }
        }

        public string Title
        {
            get { return this.ErrorMessage; }
        }

        public object Data
        {
            get { return this.ErrorMessage; }
        }

        public object PropertyObject
        {
            get { return null; }
        }

        public string ImageKey
        {
            get { return NodeImages.Keys.ErrorImage; }
        }

        public Color TextColor
        {
            get { return NodeTextColor.Public; }
        }
        #endregion IAssemblyTreeNode<TUITreeNode>

        #region IWorkspaceUnitNodeTag
        public WorkspaceUnit WorkspaceUnit { get; private set; }
        #endregion IWorkspaceUnitNodeTag

        public ErrorNodeTag(TUITreeNode treeNode, string errorMessage, WorkspaceUnit unit)
        {
            if (errorMessage == null)
                throw new ArgumentNullException("errorMessage");

            if (unit == null)
                throw new ArgumentNullException("unit");

            if (treeNode == null)
                throw new ArgumentNullException("treeNode");

            this.UITreeNode = treeNode;
            this.ErrorMessage = errorMessage;
            this.WorkspaceUnit = unit;
        }

        public string ErrorMessage { get; private set; }
    }
}
